@@ -1,22 +1,36 @@
 /* spoa-server: processing Python
  *
  * Copyright 2018 OZON / Thierry Fournier <thierry.fournier@ozon.io>
+ * Copyright (C) 2020  Gilchrist Dadaglo <gilchrist@dadaglo.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version
  * 2 of the License, or (at your option) any later version.
  *
+ * This program is provided in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
  */
 
+/*
+ *	Define PY_SSIZE_T_CLEAN before including Python.h
+ *	as per https://docs.python.org/3/c-api/arg.html and https://docs.python.org/2/c-api/arg.html
+ */
+#define PY_SSIZE_T_CLEAN
+
 #include <Python.h>
 
 #include <arpa/inet.h>
 
 #include <errno.h>
 #include <string.h>
+#include <limits.h>
 
 #include "spoa.h"
+#include "ps_python.h"
 
 /* Embedding python documentation:
  *
@@ -29,7 +43,7 @@ static PyObject *module_ipaddress;
 static PyObject *ipv4_address;
 static PyObject *ipv6_address;
 static PyObject *spoa_error;
-static PyObject *empty_tuple;
+static PyObject *empty_array;
 static struct worker *worker;
 
 static int ps_python_start_worker(struct worker *w);
@@ -43,6 +57,28 @@ static struct ps ps_python_bindings = {
 	.ext = ".py",
 };
 
+static int ps_python_check_overflow(Py_ssize_t len)
+{
+	/* There might be an overflow when converting from Py_ssize_t to int.
+	 * This function will catch those cases.
+	 * Also, spoa "struct chunk" is limited to int size.
+	 * We should not send data bigger than it can handle.
+	 */
+	if (len >= (Py_ssize_t)INT_MAX) {
+		PyErr_Format(spoa_error,
+				"%d is over 2GB. Please split in smaller pieces.", \
+				len);
+		return -1;
+	} else {
+		return Py_SAFE_DOWNCAST(len, Py_ssize_t, int);
+	}
+}
+
+#if IS_PYTHON_3K
+static PyObject *module_spoa;
+static PyObject *PyInit_spoa_module(void);
+#endif /* IS_PYTHON_3K */
+
 static PyObject *ps_python_register_message(PyObject *self, PyObject *args)
 {
 	const char *name;
@@ -50,119 +86,147 @@ static PyObject *ps_python_register_message(PyObject *self, PyObject *args)
 
 	if (!PyArg_ParseTuple(args, "sO!", &name, &PyFunction_Type, &ref))
 		return NULL;
-	Py_XINCREF(ref); /* because the function is intenally refrenced */
+	Py_XINCREF(ref); /* because the function is internally referenced */
 
 	ps_register_message(&ps_python_bindings, name, (void *)ref);
 
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_null(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
+	int name_len_i;
 	int scope;
 
 	if (!PyArg_ParseTuple(args, "s#i", &name, &name_len, &scope))
 		return NULL;
-	if (!set_var_null(worker, name, name_len, scope)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_null(worker, name, name_len_i, scope)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_boolean(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	int value;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#ii", &name, &name_len, &scope, &value))
 		return NULL;
-	if (!set_var_bool(worker, name, name_len, scope, value)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_bool(worker, name, name_len_i, scope, value)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_int32(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	int32_t value;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#ii", &name, &name_len, &scope, &value))
 		return NULL;
-	if (!set_var_int32(worker, name, name_len, scope, value)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_int32(worker, name, name_len_i, scope, value)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_uint32(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	uint32_t value;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#iI", &name, &name_len, &scope, &value))
 		return NULL;
-	if (!set_var_uint32(worker, name, name_len, scope, value)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_uint32(worker, name, name_len_i, scope, value)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_int64(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	int64_t value;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#il", &name, &name_len, &scope, &value))
 		return NULL;
-	if (!set_var_int64(worker, name, name_len, scope, value)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_int64(worker, name, name_len_i, scope, value)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_uint64(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	uint64_t value;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#ik", &name, &name_len, &scope, &value))
 		return NULL;
-	if (!set_var_uint64(worker, name, name_len, scope, value)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
+	if (!set_var_uint64(worker, name, name_len_i, scope, value)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_ipv4(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	PyObject *ipv4;
 	PyObject *value;
 	struct in_addr ip;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#iO", &name, &name_len, &scope, &ipv4))
 		return NULL;
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
 	if (!PyObject_IsInstance(ipv4, ipv4_address)) {
 		PyErr_Format(spoa_error, "must be 'IPv4Address', not '%s'", ipv4->ob_type->tp_name);
 		return NULL;
@@ -171,31 +235,33 @@ static PyObject *ps_python_set_var_ipv4(PyObject *self, PyObject *args)
 	value = PyObject_GetAttrString(ipv4, "packed");
 	if (value == NULL)
 		return NULL;
-	if (PyString_GET_SIZE(value) != sizeof(ip)) {
-		PyErr_Format(spoa_error, "IPv4 manipulation internal error");
+	if (PY_STRING_GET_SIZE(value) != sizeof(ip)) {
+		PyErr_Format(spoa_error, "UPv6 manipulation internal error");
 		return NULL;
 	}
-	memcpy(&ip, PyString_AS_STRING(value), PyString_GET_SIZE(value));
-	if (!set_var_ipv4(worker, name, name_len, scope, &ip)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	memcpy(&ip, PY_STRING_AS_STRING(value), PY_STRING_GET_SIZE(value));
+	if (!set_var_ipv4(worker, name, name_len_i, scope, &ip)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	/* Once we set the IP value in the worker, we don't need it anymore... */
-	Py_XDECREF(value);
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_ipv6(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	PyObject *ipv6;
 	PyObject *value;
 	struct in6_addr ip;
+	int name_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#iO", &name, &name_len, &scope, &ipv6))
 		return NULL;
+	name_len_i = ps_python_check_overflow(name_len);
+	if (name_len_i == -1)
+		return NULL;
 	if (!PyObject_IsInstance(ipv6, ipv6_address)) {
 		PyErr_Format(spoa_error, "must be 'IPv6Address', not '%s'", ipv6->ob_type->tp_name);
 		return NULL;
@@ -204,52 +270,62 @@ static PyObject *ps_python_set_var_ipv6(PyObject *self, PyObject *args)
 	value = PyObject_GetAttrString(ipv6, "packed");
 	if (value == NULL)
 		return NULL;
-	if (PyString_GET_SIZE(value) != sizeof(ip)) {
-		PyErr_Format(spoa_error, "IPv6 manipulation internal error");
+	if (PY_STRING_GET_SIZE(value) != sizeof(ip)) {
+		PyErr_Format(spoa_error, "UPv6 manipulation internal error");
 		return NULL;
 	}
-	memcpy(&ip, PyString_AS_STRING(value), PyString_GET_SIZE(value));
-	if (!set_var_ipv6(worker, name, name_len, scope, &ip)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	memcpy(&ip, PY_STRING_AS_STRING(value), PY_STRING_GET_SIZE(value));
+	if (!set_var_ipv6(worker, name, name_len_i, scope, &ip)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	/* Once we set the IP value in the worker, we don't need it anymore... */
-	Py_XDECREF(value);
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_str(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	const char *value;
-	int value_len;
+	Py_ssize_t value_len;
+	int name_len_i;
+	int value_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#is#", &name, &name_len, &scope, &value, &value_len))
 		return NULL;
-	if (!set_var_string(worker, name, name_len, scope, value, value_len)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	value_len_i = ps_python_check_overflow(value_len);
+	if (name_len_i == -1 || value_len_i == -1)
+		return NULL;
+	if (!set_var_string(worker, name, name_len_i, scope, value, value_len_i)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 static PyObject *ps_python_set_var_bin(PyObject *self, PyObject *args)
 {
 	const char *name;
-	int name_len;
+	Py_ssize_t name_len;
 	int scope;
 	const char *value;
-	int value_len;
+	Py_ssize_t value_len;
+	int name_len_i;
+	int value_len_i;
 
 	if (!PyArg_ParseTuple(args, "s#is#", &name, &name_len, &scope, &value, &value_len))
 		return NULL;
-	if (!set_var_bin(worker, name, name_len, scope, value, value_len)) {
-		PyErr_SetString(spoa_error, "No more memory space available");
+	name_len_i = ps_python_check_overflow(name_len);
+	value_len_i = ps_python_check_overflow(value_len);
+	if (name_len_i == -1 || value_len_i == -1)
+		return NULL;
+	if (!set_var_bin(worker, name, name_len_i, scope, value, value_len_i)) {
+		PyErr_SetString(spoa_error, "No space left available");
 		return NULL;
 	}
-	Py_RETURN_NONE;
+	return Py_None;
 }
 
 
@@ -279,6 +355,25 @@ static PyMethodDef spoa_methods[] = {
 	{ /* end */ }
 };
 
+#if IS_PYTHON_3K
+static struct PyModuleDef spoa_module_definition = {
+	PyModuleDef_HEAD_INIT,                  /* m_base     */
+	"spoa",                                 /* m_name     */
+	"HAProxy SPOA module for python",       /* m_doc      */
+	-1,                                     /* m_size     */
+	spoa_methods,                           /* m_methods  */
+	NULL,                                   /* m_slots    */
+	NULL,                                   /* m_traverse */
+	NULL,                                   /* m_clear    */
+	NULL                                    /* m_free     */
+};
+
+static PyObject *PyInit_spoa_module(void)
+{
+	return module_spoa;
+}
+#endif /* IS_PYTHON_3K */
+
 static int ps_python_start_worker(struct worker *w)
 {
 	PyObject *m;
@@ -286,10 +381,17 @@ static int ps_python_start_worker(struct worker *w)
 	PyObject *value;
 	int ret;
 
+#if IS_PYTHON_27
 	Py_SetProgramName("spoa-server");
+#endif /* IS_PYTHON_27 */
+#if IS_PYTHON_3K
+	Py_SetProgramName(Py_DecodeLocale("spoa-server", NULL));
+	PyImport_AppendInittab("spoa", &PyInit_spoa_module);
+#endif /* IS_PYTHON_3K */
+
 	Py_Initialize();
 
-	module_name = PyString_FromString("ipaddress");
+	module_name = PY_STRING_FROM_STRING("ipaddress");
 	if (module_name == NULL) {
 		PyErr_Print();
 		return 0;
@@ -304,42 +406,25 @@ static int ps_python_start_worker(struct worker *w)
 
 	ipv4_address = PyObject_GetAttrString(module_ipaddress, "IPv4Address");
 	if (ipv4_address == NULL) {
-		Py_DECREF(module_ipaddress);
 		PyErr_Print();
 		return 0;
 	}
 
 	ipv6_address = PyObject_GetAttrString(module_ipaddress, "IPv6Address");
-	if (ipv6_address == NULL) {
-		Py_DECREF(ipv4_address);
-		Py_DECREF(module_ipaddress);
+	if (ipv4_address == NULL) {
 		PyErr_Print();
 		return 0;
 	}
 
-	m = Py_InitModule("spoa", spoa_methods);
+	PY_INIT_MODULE(m, "spoa", spoa_methods, &spoa_module_definition);
 	if (m == NULL) {
-		Py_DECREF(ipv4_address);
-		Py_DECREF(ipv6_address);
-		Py_DECREF(module_ipaddress);
 		PyErr_Print();
 		return 0;
 	}
 
 	spoa_error = PyErr_NewException("spoa.error", NULL, NULL);
-	 /* PyModule_AddObject will steal the reference to spoa_error
-	 * in case of success only
-	 * We need to increment the counters to continue using it
-	 * but cleanup in case of failure
-	 */
 	Py_INCREF(spoa_error);
-	ret = PyModule_AddObject(m, "error", spoa_error);
-	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(spoa_error);
-		PyErr_Print();
-		return 0;
-	}
+	PyModule_AddObject(m, "error", spoa_error);
 
 
 	value = PyLong_FromLong(SPOE_SCOPE_PROC);
@@ -350,78 +435,67 @@ static int ps_python_start_worker(struct worker *w)
 
 	ret = PyModule_AddObject(m, "scope_proc", value);
 	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(value);
 		PyErr_Print();
 		return 0;
 	}
 
 	value = PyLong_FromLong(SPOE_SCOPE_SESS);
 	if (value == NULL) {
-		Py_DECREF(m);
 		PyErr_Print();
 		return 0;
 	}
 
 	ret = PyModule_AddObject(m, "scope_sess", value);
 	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(value);
 		PyErr_Print();
 		return 0;
 	}
 
 	value = PyLong_FromLong(SPOE_SCOPE_TXN);
 	if (value == NULL) {
-		Py_DECREF(m);
 		PyErr_Print();
 		return 0;
 	}
 
 	ret = PyModule_AddObject(m, "scope_txn", value);
 	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(value);
 		PyErr_Print();
 		return 0;
 	}
 
 	value = PyLong_FromLong(SPOE_SCOPE_REQ);
 	if (value == NULL) {
-		Py_DECREF(m);
 		PyErr_Print();
 		return 0;
 	}
 
 	ret = PyModule_AddObject(m, "scope_req", value);
 	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(value);
 		PyErr_Print();
 		return 0;
 	}
 
 	value = PyLong_FromLong(SPOE_SCOPE_RES);
 	if (value == NULL) {
-		Py_DECREF(m);
 		PyErr_Print();
 		return 0;
 	}
 
 	ret = PyModule_AddObject(m, "scope_res", value);
 	if (ret == -1) {
-		Py_DECREF(m);
-		Py_DECREF(value);
 		PyErr_Print();
 		return 0;
 	}
 
-	empty_tuple = PyTuple_New(0);
-	if (empty_tuple == NULL) {
+	empty_array = PyDict_New();
+	if (empty_array == NULL) {
 		PyErr_Print();
 		return 0;
 	}
 
+#if IS_PYTHON_3K
+	module_spoa = m;
+#endif /* IS_PYTHON_3K */
 	worker = w;
 	return 1;
 }
@@ -480,21 +554,21 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 		ent = PyDict_New();
 		if (ent == NULL) {
 			Py_DECREF(kw_args);
+			Py_DECREF(ent);
 			PyErr_Print();
 			return 0;
 		}
 
 		/* Create the name entry */
 
-		key = PyString_FromString("name");
+		key = PY_STRING_FROM_STRING("name");
 		if (key == NULL) {
 			Py_DECREF(kw_args);
-			Py_DECREF(ent);
 			PyErr_Print();
 			return 0;
 		}
 
-		value = PyString_FromStringAndSize(args[i].name.str, args[i].name.len);
+		value = PY_STRING_FROM_STRING_AND_SIZE(args[i].name.str, args[i].name.len);
 		if (value == NULL) {
 			Py_DECREF(kw_args);
 			Py_DECREF(ent);
@@ -513,9 +587,9 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 			return 0;
 		}
 
-		/* Create the value entry */
+		/* Create th value entry */
 
-		key = PyString_FromString("value");
+		key = PY_STRING_FROM_STRING("value");
 		if (key == NULL) {
 			Py_DECREF(kw_args);
 			Py_DECREF(ent);
@@ -525,7 +599,6 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 
 		switch (args[i].value.type) {
 		case SPOE_DATA_T_NULL:
-			Py_INCREF(Py_None);
 			value = Py_None;
 			break;
 		case SPOE_DATA_T_BOOL:
@@ -556,7 +629,6 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 			if (func == NULL) {
 				Py_DECREF(kw_args);
 				Py_DECREF(ent);
-				Py_DECREF(key);
 				PyErr_Print();
 				return 0;
 			}
@@ -564,16 +636,14 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 			if (ip_dict == NULL) {
 				Py_DECREF(kw_args);
 				Py_DECREF(ent);
-				Py_DECREF(key);
 				Py_DECREF(func);
 				PyErr_Print();
 				return 0;
 			}
-			ip_name = PyString_FromString("address");
+			ip_name = PY_STRING_FROM_STRING("address");
 			if (ip_name == NULL) {
 				Py_DECREF(kw_args);
 				Py_DECREF(ent);
-				Py_DECREF(key);
 				Py_DECREF(func);
 				Py_DECREF(ip_dict);
 				PyErr_Print();
@@ -583,7 +653,6 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 			if (ip_value == NULL) {
 				Py_DECREF(kw_args);
 				Py_DECREF(ent);
-				Py_DECREF(key);
 				Py_DECREF(func);
 				Py_DECREF(ip_dict);
 				Py_DECREF(ip_name);
@@ -594,27 +663,22 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 			Py_DECREF(ip_name);
 			Py_DECREF(ip_value);
 			if (ret == -1) {
-				Py_DECREF(kw_args);
-				Py_DECREF(ent);
-				Py_DECREF(key);
-				Py_DECREF(func);
 				Py_DECREF(ip_dict);
 				PyErr_Print();
 				return 0;
 			}
-			value = PyObject_Call(func, empty_tuple, ip_dict);
+			value = PyObject_Call(func, empty_array, ip_dict);
 			Py_DECREF(func);
 			Py_DECREF(ip_dict);
 			break;
 
 		case SPOE_DATA_T_STR:
-			value = PyString_FromStringAndSize(args[i].value.u.buffer.str, args[i].value.u.buffer.len);
+			value = PY_STRING_FROM_STRING_AND_SIZE(args[i].value.u.buffer.str, args[i].value.u.buffer.len);
 			break;
 		case SPOE_DATA_T_BIN:
-			value = PyString_FromStringAndSize(args[i].value.u.buffer.str, args[i].value.u.buffer.len);
+			value = PY_BYTES_FROM_STRING_AND_SIZE(args[i].value.u.buffer.str, args[i].value.u.buffer.len);
 			break;
 		default:
-			Py_INCREF(Py_None);
 			value = Py_None;
 			break;
 		}
@@ -647,7 +711,7 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 		}
 	}
 
-	/* Dictionnary { args = <list-of-args> } for the function */
+	/* Dictionary { args = <list-of-args> } for the function */
 
 	fkw = PyDict_New();
 	if (fkw == NULL) {
@@ -656,7 +720,7 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 		return 0;
 	}
 
-	key = PyString_FromString("args");
+	key = PY_STRING_FROM_STRING("args");
 	if (key == NULL) {
 		Py_DECREF(kw_args);
 		Py_DECREF(fkw);
@@ -673,13 +737,11 @@ static int ps_python_exec_message(struct worker *w, void *ref, int nargs, struct
 		return 0;
 	}
 
-	result = PyObject_Call(python_ref, empty_tuple, fkw);
-	Py_DECREF(fkw);
+	result = PyObject_Call(python_ref, empty_array, fkw);
 	if (result == NULL) {
 		PyErr_Print();
 		return 0;
 	}
-	Py_DECREF(result);
 
 	return 1;
 }
