@@ -19,7 +19,7 @@ _help()
     --v, to run verbose
       run-regtests.sh --v, disables the default vtest 'quiet' parameter
 
-    --debug to show test logs on standard ouput (implies --v)
+    --debug to show test logs on standard output (implies --v)
       run-regtests.sh --debug
 
     --keep-logs to keep all log directories (by default kept if test fails)
@@ -30,8 +30,8 @@ _help()
 
     --type <reg tests types> filter the types of the tests to be run, depending on
       the commented REGTESTS_TYPE variable value in each VTC file.
-      The value of REGTESTS_TYPE supported are: default, slow, bug, broken and
-      experimental. When not specified, it is set to 'default' as default value.
+      The value of REGTESTS_TYPE supported are: default, slow, bug, broken, devel
+      and experimental. When not specified, it is set to 'default' as default value.
 
       run-regtest.sh --type slow,default
 
@@ -63,14 +63,15 @@ _help()
     #REQUIRE_VERSION=0.0
     #REQUIRE_VERSION_BELOW=99.9
 
+    # To define required binaries for a test:
+    #REQUIRE_BINARIES=socat,curl
+
   Configure environment variables to set the haproxy and vtest binaries to use
     setenv HAPROXY_PROGRAM /usr/local/sbin/haproxy
     setenv VTEST_PROGRAM /usr/local/bin/vtest
-    setenv HAPROXY_ARGS "-dM -de -m 50"
   or
     export HAPROXY_PROGRAM=/usr/local/sbin/haproxy
     export VTEST_PROGRAM=/usr/local/bin/vtest
-    export HAPROXY_ARGS="-dM -de -m 50"
 EOF
   exit 0
 }
@@ -123,7 +124,7 @@ _startswith() {
 _findtests() {
   set -f
 
-  REGTESTS_TYPES="${REGTESTS_TYPES:-default,bug,devel,slow}"
+  REGTESTS_TYPES="${REGTESTS_TYPES:-any}"
   any_test=$(echo $REGTESTS_TYPES | grep -cw "any")
   for i in $( find "$1" -name *.vtc ); do
     skiptest=
@@ -131,6 +132,7 @@ _findtests() {
     require_version_below="$(sed -ne 's/^#REQUIRE_VERSION_BELOW=//p' "$i")"
     require_options="$(sed -ne 's/^#REQUIRE_OPTIONS=//p' "$i" | sed  -e 's/,/ /g')"
     exclude_targets="$(sed -ne 's/^#EXCLUDE_TARGETS=//p' "$i" | sed  -e 's/,/ /g')"
+    require_binaries="$(sed -ne 's/^#REQUIRE_BINARIES=//p' "$i" | sed  -e 's/,/ /g')"
     if [ $any_test -ne 1 ] ; then
         regtest_type="$(sed -ne 's/^#REGTEST_TYPE=//p' "$i")"
         if [ -z $regtest_type ] ; then
@@ -188,6 +190,14 @@ _findtests() {
       fi
     done
 
+    for requiredbin in $require_binaries; do
+      which $requiredbin >/dev/null 2>&1
+      if [ "$?" -eq "1" ]; then
+        echo "  Skip $i because '"$requiredbin"' is not installed"
+        skiptest=1
+      fi
+    done
+
     if [ -z $skiptest ]; then
       echo "  Add test: $i"
       testlist="$testlist $i"
@@ -280,7 +290,6 @@ _version() {
 
 
 HAPROXY_PROGRAM="${HAPROXY_PROGRAM:-${PWD}/haproxy}"
-HAPROXY_ARGS="${HAPROXY_ARGS--dM}"
 VTEST_PROGRAM="${VTEST_PROGRAM:-vtest}"
 TESTDIR="${TMPDIR:-/tmp}"
 REGTESTS=""
@@ -311,7 +320,7 @@ if [ $preparefailed ]; then
 fi
 
 { read HAPROXY_VERSION; read TARGET; read FEATURES; } << EOF
-$($HAPROXY_PROGRAM $HAPROXY_ARGS -vv |grep 'HA-Proxy version\|TARGET.*=\|^Feature' | sed 's/.* [:=] //')
+$($HAPROXY_PROGRAM -vv |grep 'HA-Proxy version\|TARGET.*=\|^Feature' | sed 's/.* [:=] //')
 EOF
 
 HAPROXY_VERSION=$(echo $HAPROXY_VERSION | cut -d " " -f 3)
@@ -324,9 +333,6 @@ TESTDIR=$(mktemp -d "$TESTDIR/haregtests-$TESTRUNDATETIME.XXXXXX") || exit 1
 
 export TMPDIR="$TESTDIR"
 export HAPROXY_PROGRAM="$HAPROXY_PROGRAM"
-if [ -n "$HAPROXY_ARGS" ]; then
-   export HAPROXY_ARGS
-fi
 
 echo "Target : $TARGET"
 echo "Options : $FEATURES"
