@@ -14,25 +14,20 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <common/config.h>
-#include <common/initcall.h>
-#include <common/mini-clist.h>
-#include <common/standard.h>
-#include <common/uri_auth.h>
+#include <import/ebsttree.h>
 
-#include <types/global.h>
-
-#include <proto/acl.h>
-#include <proto/arg.h>
-#include <proto/auth.h>
-#include <proto/channel.h>
-#include <proto/log.h>
-#include <proto/pattern.h>
-#include <proto/proxy.h>
-#include <proto/sample.h>
-#include <proto/stick_table.h>
-
-#include <ebsttree.h>
+#include <haproxy/acl.h>
+#include <haproxy/api.h>
+#include <haproxy/arg.h>
+#include <haproxy/auth.h>
+#include <haproxy/errors.h>
+#include <haproxy/global.h>
+#include <haproxy/list.h>
+#include <haproxy/pattern.h>
+#include <haproxy/proxy-t.h>
+#include <haproxy/sample.h>
+#include <haproxy/stick_table.h>
+#include <haproxy/tools.h>
 
 /* List head of all known ACL keywords */
 static struct acl_kw_list acl_keywords = {
@@ -90,7 +85,7 @@ struct acl_keyword *find_acl_kw(const char *kw)
 	struct acl_kw_list *kwl;
 
 	kwend = kw;
-	while (*kwend && *kwend != '(' && *kwend != ',')
+	while (is_idchar(*kwend))
 		kwend++;
 
 	list_for_each_entry(kwl, &acl_keywords.list, list) {
@@ -190,29 +185,15 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 		smp->arg_p = empty_arg_list;
 
 		/* look for the beginning of the subject arguments */
-		for (arg = args[0]; *arg && *arg != '(' && *arg != ','; arg++);
-
-		endt = arg;
-		if (*endt == '(') {
-			/* look for the end of this term and skip the opening parenthesis */
-			endt = ++arg;
-			while (*endt && *endt != ')')
-				endt++;
-			if (*endt != ')') {
-				memprintf(err, "missing closing ')' after arguments to ACL keyword '%s'", aclkw->kw);
-				goto out_free_smp;
-			}
-		}
+		for (arg = args[0]; is_idchar(*arg); arg++)
+			;
 
 		/* At this point, we have :
 		 *   - args[0] : beginning of the keyword
 		 *   - arg     : end of the keyword, first character not part of keyword
-		 *               nor the opening parenthesis (so first character of args
-		 *               if present).
-		 *   - endt    : end of the term (=arg or last parenthesis if args are present)
 		 */
-		nbargs = make_arg_list(arg, endt - arg, smp->fetch->arg_mask, &smp->arg_p,
-		                       err, NULL, NULL, al);
+		nbargs = make_arg_list(arg, -1, smp->fetch->arg_mask, &smp->arg_p,
+		                       err, &endt, NULL, al);
 		if (nbargs < 0) {
 			/* note that make_arg_list will have set <err> here */
 			memprintf(err, "ACL keyword '%s' : %s", aclkw->kw, *err);
@@ -240,9 +221,8 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 		while (*arg) {
 			struct sample_conv *conv;
 			struct sample_conv_expr *conv_expr;
-
-			if (*arg == ')') /* skip last closing parenthesis */
-				arg++;
+			int err_arg;
+			int argcnt;
 
 			if (*arg && *arg != ',') {
 				if (ckw)
@@ -254,6 +234,9 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 				goto out_free_smp;
 			}
 
+			/* FIXME: how long should we support such idiocies ? Maybe we
+			 * should already warn ?
+			 */
 			while (*arg == ',') /* then trailing commas */
 				arg++;
 
@@ -263,7 +246,8 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 				/* none ? end of converters */
 				break;
 
-			for (endw = begw; *endw && *endw != '(' && *endw != ','; endw++);
+			for (endw = begw; is_idchar(*endw); endw++)
+				;
 
 			free(ckw);
 			ckw = my_strndup(begw, endw - begw);
@@ -277,16 +261,6 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			}
 
 			arg = endw;
-			if (*arg == '(') {
-				/* look for the end of this term */
-				while (*arg && *arg != ')')
-					arg++;
-				if (*arg != ')') {
-					memprintf(err, "ACL keyword '%s' : syntax error: missing ')' after converter '%s'.",
-						  aclkw->kw, ckw);
-					goto out_free_smp;
-				}
-			}
 
 			if (conv->in_type >= SMP_TYPES || conv->out_type >= SMP_TYPES) {
 				memprintf(err, "ACL keyword '%s' : returns type of converter '%s' is unknown.",
@@ -310,35 +284,26 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 			conv_expr->conv = conv;
 			acl_conv_found = 1;
 
-			if (arg != endw) {
-				int err_arg;
-
-				if (!conv->arg_mask) {
-					memprintf(err, "ACL keyword '%s' : converter '%s' does not support any args.",
-						  aclkw->kw, ckw);
-					goto out_free_smp;
-				}
-
-				al->kw = smp->fetch->kw;
-				al->conv = conv_expr->conv->kw;
-				if (make_arg_list(endw + 1, arg - endw - 1, conv->arg_mask, &conv_expr->arg_p, err, NULL, &err_arg, al) < 0) {
-					memprintf(err, "ACL keyword '%s' : invalid arg %d in converter '%s' : %s.",
-						  aclkw->kw, err_arg+1, ckw, *err);
-					goto out_free_smp;
-				}
-
-				if (!conv_expr->arg_p)
-					conv_expr->arg_p = empty_arg_list;
-
-				if (conv->val_args && !conv->val_args(conv_expr->arg_p, conv, file, line, err)) {
-					memprintf(err, "ACL keyword '%s' : invalid args in converter '%s' : %s.",
-						  aclkw->kw, ckw, *err);
-					goto out_free_smp;
-				}
+			al->kw = smp->fetch->kw;
+			al->conv = conv_expr->conv->kw;
+			argcnt = make_arg_list(endw, -1, conv->arg_mask, &conv_expr->arg_p, err, &arg, &err_arg, al);
+			if (argcnt < 0) {
+				memprintf(err, "ACL keyword '%s' : invalid arg %d in converter '%s' : %s.",
+				          aclkw->kw, err_arg+1, ckw, *err);
+				goto out_free_smp;
 			}
-			else if (ARGM(conv->arg_mask)) {
-				memprintf(err, "ACL keyword '%s' : missing args for converter '%s'.",
-					  aclkw->kw, ckw);
+
+			if (argcnt && !conv->arg_mask) {
+				memprintf(err, "converter '%s' does not support any args", ckw);
+				goto out_free_smp;
+			}
+
+			if (!conv_expr->arg_p)
+				conv_expr->arg_p = empty_arg_list;
+
+			if (conv->val_args && !conv->val_args(conv_expr->arg_p, conv, file, line, err)) {
+				memprintf(err, "ACL keyword '%s' : invalid args in converter '%s' : %s.",
+					  aclkw->kw, ckw, *err);
 				goto out_free_smp;
 			}
 		}
@@ -351,7 +316,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 		 * so, we retrieve a completely parsed expression with args and
 		 * convs already done.
 		 */
-		smp = sample_parse_expr((char **)args, &idx, file, line, err, al);
+		smp = sample_parse_expr((char **)args, &idx, file, line, err, al, NULL);
 		if (!smp) {
 			memprintf(err, "%s in ACL expression '%s'", *err, *args);
 			goto out_return;
@@ -533,7 +498,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 		 expr->kw, file, line);
 	trash.area[trash.size - 1] = '\0';
 
-	/* Create new patern reference. */
+	/* Create new pattern reference. */
 	ref = pat_ref_newid(unique_id, trash.area, PAT_REF_ACL);
 	if (!ref) {
 		memprintf(err, "memory error");
@@ -551,7 +516,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 
 		/* Compatibility layer. Each pattern can parse only one string per pattern,
 		 * but the pat_parser_int() and pat_parse_dotted_ver() parsers were need
-		 * optionnaly two operators. The first operator is the match method: eq,
+		 * optionally two operators. The first operator is the match method: eq,
 		 * le, lt, ge and gt. pat_parse_int() and pat_parse_dotted_ver() functions
 		 * can have a compatibility syntax based on ranges:
 		 *
@@ -642,6 +607,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 
 				case STD_OP_GT:
 					value++; /* gt = ge + 1 */
+					/* fall through */
 
 				case STD_OP_GE:
 					if (expr->pat.parse == pat_parse_int)
@@ -654,6 +620,7 @@ struct acl_expr *parse_acl_expr(const char **args, char **err, struct arg_list *
 
 				case STD_OP_LT:
 					value--; /* lt = le - 1 */
+					/* fall through */
 
 				case STD_OP_LE:
 					if (expr->pat.parse == pat_parse_int)
@@ -985,7 +952,6 @@ struct acl_cond *parse_acl_cond(const char **args, struct list *known_acl,
 				/* note that parse_acl() must have filled <err> here */
 				goto out_free_suite;
 			}
-			word = args[arg + 1];
 			arg = arg_end;
 		}
 		else {
