@@ -10,32 +10,26 @@
  * 2 of the License, or (at your option) any later version.
  */
 
-#include <eb32tree.h>
+#include <import/eb32tree.h>
 #include <import/sha1.h>
 
-#include <types/action.h>
-#include <types/cli.h>
-#include <types/filters.h>
-#include <types/proxy.h>
-#include <types/shctx.h>
-
-#include <proto/channel.h>
-#include <proto/cli.h>
-#include <proto/proxy.h>
-#include <proto/http_htx.h>
-#include <proto/filters.h>
-#include <proto/http_rules.h>
-#include <proto/http_ana.h>
-#include <proto/log.h>
-#include <proto/stream.h>
-#include <proto/stream_interface.h>
-#include <proto/shctx.h>
-
-
-#include <common/cfgparse.h>
-#include <common/hash.h>
-#include <common/htx.h>
-#include <common/initcall.h>
+#include <haproxy/action-t.h>
+#include <haproxy/api.h>
+#include <haproxy/cfgparse.h>
+#include <haproxy/channel.h>
+#include <haproxy/cli.h>
+#include <haproxy/errors.h>
+#include <haproxy/filters.h>
+#include <haproxy/hash.h>
+#include <haproxy/http_ana.h>
+#include <haproxy/http_htx.h>
+#include <haproxy/http_rules.h>
+#include <haproxy/htx.h>
+#include <haproxy/net_helper.h>
+#include <haproxy/proxy.h>
+#include <haproxy/shctx.h>
+#include <haproxy/stream.h>
+#include <haproxy/stream_interface.h>
 
 #define CACHE_FLT_F_IMPLICIT_DECL  0x00000001 /* The cache filtre was implicitly declared (ie without
 					       * the filter keyword) */
@@ -95,7 +89,7 @@ struct cache_entry *entry_exist(struct cache *cache, char *hash)
 	struct eb32_node *node;
 	struct cache_entry *entry;
 
-	node = eb32_lookup(&cache->entries, (*(unsigned int *)hash));
+	node = eb32_lookup(&cache->entries, read_u32(hash));
 	if (!node)
 		return NULL;
 
@@ -199,55 +193,41 @@ cache_store_check(struct proxy *px, struct flt_conf *fconf)
 }
 
 static int
-cache_store_chn_start_analyze(struct stream *s, struct filter *filter, struct channel *chn)
+cache_store_strm_init(struct stream *s, struct filter *filter)
 {
-	if (!(chn->flags & CF_ISRESP))
-		return 1;
+	struct cache_st *st;
 
-	if (filter->ctx == NULL) {
-		struct cache_st *st;
+	st = pool_alloc_dirty(pool_head_cache_st);
+	if (st == NULL)
+		return -1;
 
-		st = pool_alloc_dirty(pool_head_cache_st);
-		if (st == NULL)
-			return -1;
-
-		st->first_block = NULL;
-		filter->ctx     = st;
-
-		/* Register post-analyzer on AN_RES_WAIT_HTTP */
-		filter->post_analyzers |= AN_RES_WAIT_HTTP;
-	}
+	st->first_block = NULL;
+	filter->ctx     = st;
 
+	/* Register post-analyzer on AN_RES_WAIT_HTTP */
+	filter->post_analyzers |= AN_RES_WAIT_HTTP;
 	return 1;
 }
 
-static int
-cache_store_chn_end_analyze(struct stream *s, struct filter *filter, struct channel *chn)
+static void
+cache_store_strm_deinit(struct stream *s, struct filter *filter)
 {
 	struct cache_st *st = filter->ctx;
 	struct cache_flt_conf *cconf = FLT_CONF(filter);
 	struct cache *cache = cconf->c.cache;
 	struct shared_context *shctx = shctx_ptr(cache);
 
-	if (!(chn->flags & CF_ISRESP))
-		return 1;
-
 	/* Everything should be released in the http_end filter, but we need to do it
 	 * there too, in case of errors */
-
 	if (st && st->first_block) {
-
 		shctx_lock(shctx);
 		shctx_row_dec_hot(shctx, st->first_block);
 		shctx_unlock(shctx);
-
 	}
 	if (st) {
 		pool_free(pool_head_cache_st, st);
 		filter->ctx = NULL;
 	}
-
-	return 1;
 }
 
 static int
@@ -533,7 +513,7 @@ static void cache_free_blocks(struct shared_block *first, struct shared_block *b
 }
 
 /*
- * This fonction will store the headers of the response in a buffer and then
+ * This function will store the headers of the response in a buffer and then
  * register a filter to store the data
  */
 enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
@@ -549,7 +529,7 @@ enum act_return http_action_store_cache(struct act_rule *rule, struct proxy *px,
 	struct shared_context *shctx = shctx_ptr(cconf->c.cache);
 	struct cache_st *cache_ctx = NULL;
 	struct cache_entry *object, *old;
-	unsigned int key = *(unsigned int *)txn->cache_hash;
+	unsigned int key = read_u32(txn->cache_hash);
 	struct htx *htx;
 	struct http_hdr_ctx ctx;
 	size_t hdrs_len = 0;
@@ -835,7 +815,7 @@ static size_t htx_cache_dump_msg(struct appctx *appctx, struct htx *htx, unsigne
 			goto add_data_blk;
 		}
 
-		/* Get info of the next HTX block. May be splitted on 2 shblk */
+		/* Get info of the next HTX block. May be split on 2 shblk */
 		sz = MIN(4, shctx->block_size - offset);
 		memcpy((char *)&info, (const char *)shblk->data + offset, sz);
 		offset += sz;
@@ -901,7 +881,7 @@ static void http_cache_io_handler(struct appctx *appctx)
 	if (unlikely(si->state == SI_ST_DIS || si->state == SI_ST_CLO))
 		goto out;
 
-	/* Check if the input buffer is avalaible. */
+	/* Check if the input buffer is available. */
 	if (!b_size(&res->buf)) {
 		si_rx_room_blk(si);
 		goto out;
@@ -946,6 +926,7 @@ static void http_cache_io_handler(struct appctx *appctx)
 	}
 
 	if (appctx->st0 == HTX_CACHE_EOM) {
+		res_htx->flags |= HTX_FL_EOI; /* no more data are expected. Only EOM remains to add now */
 		if (!htx_add_endof(res_htx, HTX_BLK_EOM)) {
 			si_rx_room_blk(si);
 			goto out;
@@ -1400,9 +1381,11 @@ struct flt_ops cache_ops = {
 	.check  = cache_store_check,
 	.deinit = cache_store_deinit,
 
+	/* Handle stream init/deinit */
+	.attach = cache_store_strm_init,
+	.detach = cache_store_strm_deinit,
+
 	/* Handle channels activity */
-	.channel_start_analyze = cache_store_chn_start_analyze,
-	.channel_end_analyze = cache_store_chn_end_analyze,
 	.channel_post_analyze = cache_store_post_analyze,
 
 	/* Filter HTTP requests and responses */
@@ -1528,7 +1511,7 @@ static int cli_io_handler_show_cache(struct appctx *appctx)
 			}
 
 			entry = container_of(node, struct cache_entry, eb);
-			chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n", entry, (*(unsigned int *)entry->hash), block_ptr(entry)->len, block_ptr(entry)->block_count, block_ptr(entry)->refcount, entry->expire - (int)now.tv_sec);
+			chunk_printf(&trash, "%p hash:%u size:%u (%u blocks), refcount:%u, expire:%d\n", entry, read_u32(entry->hash), block_ptr(entry)->len, block_ptr(entry)->block_count, block_ptr(entry)->refcount, entry->expire - (int)now.tv_sec);
 
 			next_key = node->key + 1;
 			appctx->ctx.cli.i0 = next_key;
