@@ -11,9 +11,9 @@
  */
 
 #include <ctype.h>
-#include <common/config.h>
-#include <common/http.h>
-#include <common/standard.h>
+#include <haproxy/api.h>
+#include <haproxy/http.h>
+#include <haproxy/tools.h>
 
 /* It is about twice as fast on recent architectures to lookup a byte in a
  * table than to perform a boolean AND or OR between two tests. Refer to
@@ -155,74 +155,14 @@ const unsigned char http_char_classes[256] = {
 	[127] = HTTP_FLG_CTL,
 };
 
-/* We must put the messages here since GCC cannot initialize consts depending
- * on strlen().
- */
-struct buffer http_err_chunks[HTTP_ERR_SIZE];
-
-const struct ist HTTP_100 = IST("HTTP/1.1 100 Continue\r\n\r\n");
-
-const struct ist HTTP_103 = IST("HTTP/1.1 103 Early Hints\r\n");
-
-/* Warning: no "connection" header is provided with the 3xx messages below */
-const char *HTTP_301 =
-	"HTTP/1.1 301 Moved Permanently\r\n"
-	"Content-length: 0\r\n"
-	"Location: "; /* not terminated since it will be concatenated with the URL */
-
-const char *HTTP_302 =
-	"HTTP/1.1 302 Found\r\n"
-	"Cache-Control: no-cache\r\n"
-	"Content-length: 0\r\n"
-	"Location: "; /* not terminated since it will be concatenated with the URL */
-
-/* same as 302 except that the browser MUST retry with the GET method */
-const char *HTTP_303 =
-	"HTTP/1.1 303 See Other\r\n"
-	"Cache-Control: no-cache\r\n"
-	"Content-length: 0\r\n"
-	"Location: "; /* not terminated since it will be concatenated with the URL */
-
-/* same as 302 except that the browser MUST retry with the same method */
-const char *HTTP_307 =
-	"HTTP/1.1 307 Temporary Redirect\r\n"
-	"Cache-Control: no-cache\r\n"
-	"Content-length: 0\r\n"
-	"Location: "; /* not terminated since it will be concatenated with the URL */
-
-/* same as 301 except that the browser MUST retry with the same method */
-const char *HTTP_308 =
-	"HTTP/1.1 308 Permanent Redirect\r\n"
-	"Content-length: 0\r\n"
-	"Location: "; /* not terminated since it will be concatenated with the URL */
-
-/* Warning: this one is an sprintf() fmt string, with <realm> as its only argument */
-const char *HTTP_401_fmt =
-	"HTTP/1.1 401 Unauthorized\r\n"
-	"Content-length: 112\r\n"
-	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
-	"Content-Type: text/html\r\n"
-	"WWW-Authenticate: Basic realm=\"%s\"\r\n"
-	"\r\n"
-	"<html><body><h1>401 Unauthorized</h1>\nYou need a valid user and password to access this content.\n</body></html>\n";
-
-const char *HTTP_407_fmt =
-	"HTTP/1.1 407 Unauthorized\r\n"
-	"Content-length: 112\r\n"
-	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
-	"Content-Type: text/html\r\n"
-	"Proxy-Authenticate: Basic realm=\"%s\"\r\n"
-	"\r\n"
-	"<html><body><h1>407 Unauthorized</h1>\nYou need a valid user and password to access this content.\n</body></html>\n";
-
 const int http_err_codes[HTTP_ERR_SIZE] = {
 	[HTTP_ERR_200] = 200,  /* used by "monitor-uri" */
 	[HTTP_ERR_400] = 400,
+	[HTTP_ERR_401] = 401,
 	[HTTP_ERR_403] = 403,
 	[HTTP_ERR_404] = 404,
 	[HTTP_ERR_405] = 405,
+	[HTTP_ERR_407] = 407,
 	[HTTP_ERR_408] = 408,
 	[HTTP_ERR_410] = 410,
 	[HTTP_ERR_413] = 413,
@@ -240,7 +180,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 200 OK\r\n"
 	"Content-length: 58\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>200 OK</h1>\nService ready.\n</body></html>\n",
@@ -254,11 +193,18 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"\r\n"
 	"<html><body><h1>400 Bad request</h1>\nYour browser sent an invalid request.\n</body></html>\n",
 
+	[HTTP_ERR_401] =
+	"HTTP/1.1 401 Unauthorized\r\n"
+	"Content-length: 112\r\n"
+	"Cache-Control: no-cache\r\n"
+	"Content-Type: text/html\r\n"
+	"\r\n"
+	"<html><body><h1>401 Unauthorized</h1>\nYou need a valid user and password to access this content.\n</body></html>\n",
+
 	[HTTP_ERR_403] =
 	"HTTP/1.1 403 Forbidden\r\n"
 	"Content-length: 93\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>403 Forbidden</h1>\nRequest forbidden by administrative rules.\n</body></html>\n",
@@ -267,7 +213,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 404 Not Found\r\n"
 	"Content-length: 83\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>404 Not Found</h1>\nThe resource could not be found.\n</body></html>\n",
@@ -276,11 +221,18 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 405 Method Not Allowed\r\n"
 	"Content-length: 146\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>405 Method Not Allowed</h1>\nA request was made of a resource using a request method not supported by that resource\n</body></html>\n",
 
+	[HTTP_ERR_407] =
+	"HTTP/1.1 407 Unauthorized\r\n"
+	"Content-length: 112\r\n"
+	"Cache-Control: no-cache\r\n"
+	"Content-Type: text/html\r\n"
+	"\r\n"
+	"<html><body><h1>407 Unauthorized</h1>\nYou need a valid user and password to access this content.\n</body></html>\n",
+
 	[HTTP_ERR_408] =
 	"HTTP/1.1 408 Request Time-out\r\n"
 	"Content-length: 110\r\n"
@@ -294,7 +246,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 410 Gone\r\n"
 	"Content-length: 114\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>410 Gone</h1>\nThe resource is no longer available and will not be available again.\n</body></html>\n",
@@ -303,7 +254,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 413 Payload Too Large\r\n"
 	"Content-length: 106\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>413 Payload Too Large</h1>\nThe request entity exceeds the maximum allowed.\n</body></html>\n",
@@ -312,7 +262,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 421 Misdirected Request\r\n"
 	"Content-length: 104\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>421 Misdirected Request</h1>\nRequest sent to a non-authoritative server.\n</body></html>\n",
@@ -321,7 +270,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 425 Too Early\r\n"
 	"Content-length: 80\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>425 Too Early</h1>\nYour browser sent early data.\n</body></html>\n",
@@ -330,7 +278,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 429 Too Many Requests\r\n"
 	"Content-length: 117\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>429 Too Many Requests</h1>\nYou have sent too many requests in a given amount of time.\n</body></html>\n",
@@ -339,16 +286,14 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 500 Internal Server Error\r\n"
 	"Content-length: 96\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
-	"<html><body><h1>500 Internal Server Error</h1>\nAn internal server error occured.\n</body></html>\n",
+	"<html><body><h1>500 Internal Server Error</h1>\nAn internal server error occurred.\n</body></html>\n",
 
 	[HTTP_ERR_502] =
 	"HTTP/1.1 502 Bad Gateway\r\n"
 	"Content-length: 107\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>502 Bad Gateway</h1>\nThe server returned an invalid or incomplete response.\n</body></html>\n",
@@ -357,7 +302,6 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 503 Service Unavailable\r\n"
 	"Content-length: 107\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>503 Service Unavailable</h1>\nNo server is available to handle this request.\n</body></html>\n",
@@ -366,11 +310,9 @@ const char *http_err_msgs[HTTP_ERR_SIZE] = {
 	"HTTP/1.1 504 Gateway Time-out\r\n"
 	"Content-length: 92\r\n"
 	"Cache-Control: no-cache\r\n"
-	"Connection: close\r\n"
 	"Content-Type: text/html\r\n"
 	"\r\n"
 	"<html><body><h1>504 Gateway Time-out</h1>\nThe server didn't respond in time.\n</body></html>\n",
-
 };
 
 const struct ist http_known_methods[HTTP_METH_OTHER] = {
@@ -411,9 +353,11 @@ int http_get_status_idx(unsigned int status)
 	switch (status) {
 	case 200: return HTTP_ERR_200;
 	case 400: return HTTP_ERR_400;
+	case 401: return HTTP_ERR_401;
 	case 403: return HTTP_ERR_403;
 	case 404: return HTTP_ERR_404;
 	case 405: return HTTP_ERR_405;
+	case 407: return HTTP_ERR_407;
 	case 408: return HTTP_ERR_408;
 	case 410: return HTTP_ERR_410;
 	case 413: return HTTP_ERR_413;
@@ -449,7 +393,7 @@ const char *http_get_reason(unsigned int status)
 	case 226: return "IM Used";
 	case 300: return "Multiple Choices";
 	case 301: return "Moved Permanently";
-	case 302: return "Found";
+	case 302: return "Moved Temporarily";
 	case 303: return "See Other";
 	case 304: return "Not Modified";
 	case 305: return "Use Proxy";
@@ -495,7 +439,7 @@ const char *http_get_reason(unsigned int status)
 	case 503: return "Service Unavailable";
 	case 504: return "Gateway Time-out";
 	case 505: return "HTTP Version not supported";
-	case 506: return "Variant also negociate";
+	case 506: return "Variant also negotiate";
 	case 507: return "Insufficient storage";
 	case 508: return "Loop detected";
 	case 509: return "Bandwidth Limit Exceeded";
@@ -514,6 +458,62 @@ const char *http_get_reason(unsigned int status)
 	}
 }
 
+/* Parse the uri and looks for the authority, between the scheme and the
+ * path. if no_userinfo is not zero, the part before the '@' (including it) is
+ * skipped. If not found, an empty ist is returned. Otherwise, the ist pointing
+ * on the authority is returned.
+ */
+struct ist http_get_authority(const struct ist uri, int no_userinfo)
+{
+	const char *ptr, *start, *end;
+
+	if (!uri.len)
+		goto not_found;
+
+	ptr = uri.ptr;
+	start = ptr;
+	end = ptr + uri.len;
+
+	/* RFC7230, par. 2.7 :
+	 * Request-URI = "*" | absuri | abspath | authority
+	 */
+
+	if (*ptr == '*' || *ptr == '/')
+		goto not_found;
+
+	if (isalpha((unsigned char)*ptr)) {
+		/* this is a scheme as described by RFC3986, par. 3.1, or only
+		 * an authority (in case of a CONNECT method).
+		 */
+		ptr++;
+		while (ptr < end &&
+		       (isalnum((unsigned char)*ptr) || *ptr == '+' || *ptr == '-' || *ptr == '.'))
+			ptr++;
+		/* skip '://' or take the whole as authority if not found */
+		if (ptr == end || *ptr++ != ':')
+			goto authority;
+		if (ptr == end || *ptr++ != '/')
+			goto authority;
+		if (ptr == end || *ptr++ != '/')
+			goto authority;
+	}
+
+	start = ptr;
+	while (ptr < end && *ptr != '/') {
+		if (*ptr++ == '@' && no_userinfo)
+			start = ptr;
+	}
+
+	/* OK, ptr point on the '/' or the end */
+	end = ptr;
+
+  authority:
+	return ist2(start, end - start);
+
+  not_found:
+	return IST_NULL;
+}
+
 /* Parse the URI from the given transaction (which is assumed to be in request
  * phase) and look for the "/" beginning the PATH. If not found, ist2(0,0) is
  * returned. Otherwise the pointer and length are returned.
@@ -561,7 +561,7 @@ struct ist http_get_path(const struct ist uri)
 	return ist2(ptr, end - ptr);
 
  not_found:
-	return ist2(NULL, 0);
+	return IST_NULL;
 }
 
 /*
@@ -600,9 +600,7 @@ char *http_find_hdr_value_end(char *s, const char *e)
 
 	quoted = qdpair = 0;
 
-#if defined(__x86_64__) ||						\
-    defined(__i386__) || defined(__i486__) || defined(__i586__) || defined(__i686__) || \
-    defined(__ARM_ARCH_7A__)
+#ifdef HA_UNALIGNED_LE
 	/* speedup: skip everything not a comma nor a double quote */
 	for (; s <= e - sizeof(int); s += sizeof(int)) {
 		unsigned int c = *(int *)s; // comma
@@ -653,7 +651,6 @@ char *http_find_cookie_value_end(char *s, const char *e)
 }
 
 /* Try to find the next occurrence of a cookie name in a cookie header value.
- * To match on any cookie name, <cookie_name_l> must be set to 0.
  * The lookup begins at <hdr>. The pointer and size of the next occurrence of
  * the cookie value is returned into *value and *value_l, and the function
  * returns a pointer to the next pointer to search from if the value was found.
@@ -694,7 +691,7 @@ char *http_extract_cookie_value(char *hdr, const char *hdr_end,
 			att_end = equal;
 		}
 
-		/* here, <equal> points to '=', a delimitor or the end. <att_end>
+		/* here, <equal> points to '=', a delimiter or the end. <att_end>
 		 * is between <att_beg> and <equal>, both may be identical.
 		 */
 
@@ -708,7 +705,7 @@ char *http_extract_cookie_value(char *hdr, const char *hdr_end,
 			/* find the end of the value, respecting quotes */
 			next = http_find_cookie_value_end(val_beg, hdr_end);
 
-			/* make val_end point to the first white space or delimitor after the value */
+			/* make val_end point to the first white space or delimiter after the value */
 			val_end = next;
 			while (val_end > val_beg && HTTP_IS_SPHT(*(val_end - 1)))
 				val_end--;
@@ -731,8 +728,8 @@ char *http_extract_cookie_value(char *hdr, const char *hdr_end,
 		 * its value between val_beg and val_end.
 		 */
 
-		if (cookie_name_l == 0 || (att_end - att_beg == cookie_name_l &&
-		    memcmp(att_beg, cookie_name, cookie_name_l) == 0)) {
+		if (att_end - att_beg == cookie_name_l &&
+		    memcmp(att_beg, cookie_name, cookie_name_l) == 0) {
 			/* let's return this value and indicate where to go on from */
 			*value = val_beg;
 			*value_l = val_end - val_beg;
@@ -1013,29 +1010,40 @@ int http_parse_stline(const struct ist line, struct ist *p1, struct ist *p2, str
         /* Skip spaces between p2 and p3 */
         for (; p < end && HTTP_IS_SPHT(*p); p++);
 
-        /* The remaing is the third value */
+        /* The remaining is the third value */
         p3->ptr = p;
         p3->len = end - p;
 
         return 1;
 }
 
-
-/* post-initializes the HTTP parts. Returns zero on error, with <err>
- * pointing to the error message.
+/* Parses value of a Status header with the following format: "Status: Code[
+ * Reason]".  The parsing is pretty naive and just skip spaces. It return the
+ * numeric value of the status code.
  */
-int init_http(char **err)
+int http_parse_status_val(const struct ist value, struct ist *status, struct ist *reason)
 {
-	int msg;
+	char *p   = value.ptr;
+        char *end = p + value.len;
+	uint16_t code;
 
-	for (msg = 0; msg < HTTP_ERR_SIZE; msg++) {
-		if (!http_err_msgs[msg]) {
-			memprintf(err, "Internal error: no message defined for HTTP return code %d", msg);
-			return 0;
-		}
+	status->len = reason->len = 0;
 
-		http_err_chunks[msg].area = (char *)http_err_msgs[msg];
-		http_err_chunks[msg].data = strlen(http_err_msgs[msg]);
-	}
-	return 1;
+	/* Skip leading spaces */
+        for (; p < end && HTTP_IS_SPHT(*p); p++);
+
+        /* Set the status part */
+        status->ptr = p;
+        for (; p < end && HTTP_IS_TOKEN(*p); p++);
+        status->len = p - status->ptr;
+
+	/* Skip spaces between status and reason */
+        for (; p < end && HTTP_IS_SPHT(*p); p++);
+
+	/* the remaining is the reason */
+        reason->ptr = p;
+        reason->len = end - p;
+
+	code = strl2ui(status->ptr, status->len);
+	return code;
 }
