@@ -10,8 +10,6 @@
  *
  */
 
-#define _GNU_SOURCE
-
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -19,31 +17,27 @@
 #include <string.h>
 #include <sys/wait.h>
 
-#include <common/cfgparse.h>
-#include <common/initcall.h>
-#include <common/mini-clist.h>
-#include <common/version.h>
-
-#include <types/cli.h>
-#include <types/global.h>
-#include <types/peers.h>
-#include <types/signal.h>
-
-#include <proto/cli.h>
-#include <proto/fd.h>
-#include <proto/listener.h>
-#include <proto/log.h>
-#include <proto/mworker.h>
-#include <proto/proxy.h>
-#include <proto/signal.h>
-#include <proto/stream.h>
-#include <proto/stream_interface.h>
-
-
 #if defined(USE_SYSTEMD)
 #include <systemd/sd-daemon.h>
 #endif
 
+#include <haproxy/api.h>
+#include <haproxy/cfgparse.h>
+#include <haproxy/cli.h>
+#include <haproxy/errors.h>
+#include <haproxy/fd.h>
+#include <haproxy/global.h>
+#include <haproxy/list.h>
+#include <haproxy/listener.h>
+#include <haproxy/mworker.h>
+#include <haproxy/peers.h>
+#include <haproxy/proxy-t.h>
+#include <haproxy/signal.h>
+#include <haproxy/stream.h>
+#include <haproxy/stream_interface.h>
+#include <haproxy/version.h>
+
+
 static int exitcode = -1;
 static int max_reloads = -1; /* number max of reloads a worker can have until they are killed */
 
@@ -133,13 +127,13 @@ void mworker_proc_list_to_env()
 /*
  * unserialize the proc list from the environment
  */
-int mworker_env_to_proc_list()
+void mworker_env_to_proc_list()
 {
 	char *msg, *token = NULL, *s1;
 
 	msg = getenv("HAPROXY_PROCESSES");
 	if (!msg)
-		return 0;
+		return;
 
 	while ((token = strtok_r(msg, "|", &s1))) {
 		struct mworker_proc *child;
@@ -149,10 +143,6 @@ int mworker_env_to_proc_list()
 		msg = NULL;
 
 		child = calloc(1, sizeof(*child));
-		if (!child) {
-			ha_alert("Out of memory while trying to allocate a worker process structure.");
-			return -1;
-		}
 
 		while ((subtoken = strtok_r(token, ";", &s2))) {
 
@@ -173,8 +163,6 @@ int mworker_env_to_proc_list()
 
 			} else if (strncmp(subtoken, "fd=", 3) == 0) {
 				child->ipc_fd[0] = atoi(subtoken+3);
-				if (child->ipc_fd[0] > -1)
-					global.maxsock++;
 			} else if (strncmp(subtoken, "pid=", 4) == 0) {
 				child->pid = atoi(subtoken+4);
 			} else if (strncmp(subtoken, "rpid=", 5) == 0) {
@@ -201,8 +189,6 @@ int mworker_env_to_proc_list()
 	}
 
 	unsetenv("HAPROXY_PROCESSES");
-
-	return 0;
 }
 
 /* Signal blocking and unblocking */
@@ -300,8 +286,12 @@ restart_wait:
 		} else {
 			/* check if exited child is a current child */
 			if (!(child->options & PROC_O_LEAVING)) {
-				if (child->options & PROC_O_TYPE_WORKER)
-					ha_alert("Current worker #%d (%d) exited with code %d (%s)\n", child->relative_pid, exitpid, status, (status >= 128) ? strsignal(status - 128) : "Exit");
+				if (child->options & PROC_O_TYPE_WORKER) {
+					if (status < 128)
+						ha_warning("Current worker #%d (%d) exited with code %d (%s)\n", child->relative_pid, exitpid, status, "Exit");
+					else
+						ha_alert("Current worker #%d (%d) exited with code %d (%s)\n", child->relative_pid, exitpid, status, strsignal(status - 128));
+				}
 				else if (child->options & PROC_O_TYPE_PROG)
 					ha_alert("Current program '%s' (%d) exited with code %d (%s)\n", child->id, exitpid, status, (status >= 128) ? strsignal(status - 128) : "Exit");
 
@@ -390,9 +380,6 @@ static int mworker_pipe_register_per_thread()
 	if (tid != 0)
 		return 1;
 
-	if (proc_self->ipc_fd[1] < 0) /* proc_self was incomplete and we can't find the socketpair */
-		return 1;
-
 	fcntl(proc_self->ipc_fd[1], F_SETFL, O_NONBLOCK);
 	/* In multi-tread, we need only one thread to process
 	 * events on the pipe with master
@@ -422,10 +409,8 @@ void mworker_cleanlisteners()
 
 		stop_proxy(curpeers->peers_fe);
 		/* disable this peer section so that it kills itself */
-		if (curpeers->sighandler)
-			signal_unregister_handler(curpeers->sighandler);
-		if (curpeers->sync_task)
-			task_destroy(curpeers->sync_task);
+		signal_unregister_handler(curpeers->sighandler);
+		task_destroy(curpeers->sync_task);
 		curpeers->sync_task = NULL;
 		task_destroy(curpeers->peers_fe->task);
 		curpeers->peers_fe->task = NULL;
@@ -456,47 +441,18 @@ void mworker_cleanlisteners()
 	}
 }
 
-/* Upon a configuration loading error some mworker_proc and FDs/server were
- * assigned but the worker was never forked, we must close the FDs and
- * remove the server
- */
-void mworker_cleanup_proc()
-{
-	struct mworker_proc *child, *it;
-
-	list_for_each_entry_safe(child, it, &proc_list, list) {
-
-		if (child->pid == -1) {
-			/* Close the socketpair master side.  We don't need to
-			 * close the worker side, because it's stored in the
-			 * GLOBAL cli listener which was supposed to be in the
-			 * worker and which will be closed in
-			 * mworker_cleanlisteners()
-			 */
-			if (child->ipc_fd[0] > -1)
-				close(child->ipc_fd[0]);
-			LIST_DEL(&child->list);
-			mworker_free_child(child);
-		}
-	}
-}
-
-
 /*  Displays workers and processes  */
 static int cli_io_handler_show_proc(struct appctx *appctx)
 {
 	struct stream_interface *si = appctx->owner;
 	struct mworker_proc *child;
 	int old = 0;
-	int up = date.tv_sec - proc_self->timestamp;
+	int up = now.tv_sec - proc_self->timestamp;
 	char *uptime = NULL;
 
 	if (unlikely(si_ic(si)->flags & (CF_WRITE_ERROR|CF_SHUTW)))
 		return 1;
 
-	if (up < 0) /* must never be negative because of clock drift */
-		up = 0;
-
 	chunk_reset(&trash);
 
 	chunk_printf(&trash, "#%-14s %-15s %-15s %-15s %-15s %-15s\n", "<PID>", "<type>", "<relative PID>", "<reloads>", "<uptime>", "<version>");
@@ -509,9 +465,7 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 
 	chunk_appendf(&trash, "# workers\n");
 	list_for_each_entry(child, &proc_list, list) {
-		up = date.tv_sec - child->timestamp;
-		if (up < 0) /* must never be negative because of clock drift */
-			up = 0;
+		up = now.tv_sec - child->timestamp;
 
 		if (!(child->options & PROC_O_TYPE_WORKER))
 			continue;
@@ -533,9 +487,7 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 
 		chunk_appendf(&trash, "# old workers\n");
 		list_for_each_entry(child, &proc_list, list) {
-			up = date.tv_sec - child->timestamp;
-			if (up <= 0) /* must never be negative because of clock drift */
-				up = 0;
+			up = now.tv_sec - child->timestamp;
 
 			if (!(child->options & PROC_O_TYPE_WORKER))
 				continue;
@@ -555,9 +507,7 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 	chunk_appendf(&trash, "# programs\n");
 	old = 0;
 	list_for_each_entry(child, &proc_list, list) {
-		up = date.tv_sec - child->timestamp;
-		if (up < 0) /* must never be negative because of clock drift */
-			up = 0;
+		up = now.tv_sec - child->timestamp;
 
 		if (!(child->options & PROC_O_TYPE_PROG))
 			continue;
@@ -575,9 +525,7 @@ static int cli_io_handler_show_proc(struct appctx *appctx)
 	if (old) {
 		chunk_appendf(&trash, "# old programs\n");
 		list_for_each_entry(child, &proc_list, list) {
-			up = date.tv_sec - child->timestamp;
-			if (up < 0) /* must never be negative because of clock drift */
-				up = 0;
+			up = now.tv_sec - child->timestamp;
 
 			if (!(child->options & PROC_O_TYPE_PROG))
 				continue;
